#include <iostream>
#include <vector>

using namespace std;

class Tree {
public:
	virtual void add(Tree*) = 0;
	virtual void print() = 0;
};

class BaseNode : public Tree {
public:
	BaseNode(int v) : value(v) {};
	virtual void print() {
		cout << " " << value << " ";
	};
	virtual void add(Tree*) {};      
private:
	int value;
};

class IntermediateNode : public Tree {
public:
	IntermediateNode(int v) : value(v) {};
	virtual void add(Tree*);
	virtual void print(); 
private:
	int value;
	vector<Tree*> next;
};

void IntermediateNode::add(Tree* t){
	next.push_back(t);
}

void IntermediateNode::print(){
	cout << "-" << value << "[";
	vector<Tree*>:: iterator it;
	
	for (it = next.begin(); it != next.end(); ++it)
		(*it)->print();
	cout << "]";
}


int main(){
	
	Tree* t = new IntermediateNode(10);
	Tree* b = new BaseNode(5);
	t->add(new BaseNode(5));
	Tree* l1 = new IntermediateNode(20);
	l1->add(new BaseNode(67));
	l1->add(new BaseNode(20));
	t->add(l1);
	t->print();
	cout<<endl;    
	// delete allocated memory - what about the anonymous allocations? 
   delete l1;
   delete b;
   delete t;
  // This does not delete the anonymous allocations -> Deletion needs to be done by composite
  // Implication is that the composite must implement destructors and that the base class  destructor MUST be virtual 



	return 0;
}
